/*
* Licensed Materials - Property of IBM
* 5725-G92 (C) Copyright IBM Corp. 2006, 2013. All Rights Reserved.
* US Government Users Restricted Rights - Use, duplication or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/

/*******************************************************************************/
var __integration = new com.worklight.integration.js.JavaScriptIntegrationLibraryImplementation();

/*******************************************************************************/

var WL = WL ? WL : {};
WL.Server = WL.Server ? WL.Server : {};
WL.Logger = WL.Logger ? WL.Logger : {};

// pretty printing breaks digital signature value
XML.prettyPrinting = false;
XML.ignoreWhitespace = false;

/*******************************************************************************/
WL.Server.configuration = __integration.getWorklightConfiguration();

/*******************************************************************************/
WL.Server.getActiveUser = function(realm) {
    return __integration.getUserIdentity(realm ? realm.toString() : null);
};

/*******************************************************************************/
WL.Server.getClientRequest = function() {
    return com.worklight.core.auth.impl.AuthenticationContext.getCurrentRequest();
};

/*******************************************************************************/
WL.Server.setActiveUser = function(realm, identity) {
    return __integration.setUserIdentity(realm.toString(), identity);
};

/*******************************************************************************/
WL.Logger.log = function(value) {
    var str = typeof value == 'object' ? __toString(value) : value;
    __integration.log(str);
}

/*******************************************************************************/
WL.Logger.debug = function(value) {
    var str = typeof value == 'object' ? __toString(value) : value;
    __integration.debug(str);
}

/*******************************************************************************/
WL.Logger.error = function(value) {
    var str = typeof value == 'object' ? __toString(value) : value;
    __integration.error(str);
}

/*******************************************************************************/
WL.Server.invokeProcedure = function(invocationData) {
	var res = __validateInvocationData(invocationData);
	if (res) return res;
	var qname = new com.worklight.server.integration.api.ProcedureQName(invocationData.adapter, invocationData.procedure);
	var procedure = __integration.getAdapterManager().getProcedure(qname);
	var requestId = __integration.getCurrentRequestId();
	var parameters = invocationData.parameters;
	if (parameters == undefined) {
		parameters = {};
	} else if (typeof parameters == 'string') {
		throw "Parameters cannot be a string, It must be an object."; 
	} 
	var javaIR = __getDataAccessService().invokeProcedureInternal(procedure, parameters, requestId);
	return javaIR.toScriptable();
}

/*******************************************************************************/
WL.Server.invokeHttp = function(options) {
	if (options) {
		if (typeof options.parameters == 'string') {
			throw "Parameters cannot be a string, It must be an object.";
		}
	}
    return __integration.invokeDynamicProcedure("HttpRequest", new com.worklight.adapters.http.HTTP(), [options]);
}

/*******************************************************************************/
WL.Server.invokeCastIron = function(options) {
	
	if (options) {
		if (typeof options.parameters == 'string') {
			throw "Parameters cannot be a string, It must be an object.";
		}
		var appNameParm = options['appName'];
		if(appNameParm){
			var headerOpt = options['headers'];
			if(headerOpt){
				if(! headerOpt["X-WL-APP"]){
					headerOpt["X-WL-APP"] = appNameParm;
				}
			}else{
				options['headers'] = {"X-WL-APP": appNameParm};
			}
			delete options['appName'];
		}else{
			throw "Required parameter 'appName' was missing";
		}
		
	}else{
		throw "Options are required for this function";
	}
    return __integration.invokeDynamicProcedure("CastIronRequest", new com.worklight.adapters.ci.CastIron(), [options]);
}


/*******************************************************************************/
WL.Server.createSQLStatement = function(options) {
    return __integration.createSQLStatement(options);
}

/*******************************************************************************/
WL.Server.invokeSQLStatement = function(options) {
    return __integration.invokeDynamicProcedure("SqlStatement", new com.worklight.adapters.sql.SQLQuery(), [options]);
}

/*******************************************************************************/
WL.Server.invokeSQLStoredProcedure = function(options) {
    return __integration.invokeDynamicProcedure("StoredProcedure", new com.worklight.adapters.sql.SQLStoredProcedure(), [options]);
}

/*******************************************************************************/
WL.Server.readSingleJMSMessage = function(options) {
    options.singleMessage=true;
    return __integration.invokeDynamicProcedure("JMSConsumer", new com.worklight.adapters.jms.JMSSynchronousConsumer(), [options]);
}

/*******************************************************************************/
WL.Server.readAllJMSMessages = function(options) {
    options.singleMessage=false;
    return __integration.invokeDynamicProcedure("JMSConsumer", new com.worklight.adapters.jms.JMSSynchronousConsumer(), [options]);
}

/*******************************************************************************/
WL.Server.writeJMSMessage = function(options) {
    return __integration.invokeDynamicProcedure("JMSProducer", new com.worklight.adapters.jms.JMSSynchronousProducer(), [options]);
}

/*******************************************************************************/
WL.Server.requestReplyJMSMessage = function(options) {
    return __integration.invokeDynamicProcedure("JMSRequestReply", new com.worklight.adapters.jms.JMSSynchronousProducerConsumer(), [options]);
}

/*******************************************************************************/
WL.Server.createEventSource = function(options) {
    return __integration.createEventSource(options);
}

/*******************************************************************************/
WL.Server.getUserNotificationSubscription = function(eventSource, userId) {
    var userSubscription = __integration.getUserSubscription(eventSource, userId);
    if (userSubscription != null) {
        userSubscription.save = function() {
            this.delegate.save(this);
        }
        userSubscription.getDeviceSubscriptions = function() {
            return this.delegate.getDeviceSubscriptions();
        }
    }
    return userSubscription;
}

/*******************************************************************************/
WL.Server.notifyAllDevices = function(userSubscription, options) {
	var wasDeprecationWarningMessagePrinted = false;
    var devices = userSubscription.getDeviceSubscriptions();
    for (var i = 0; i  < devices.length; i++) {
        WL.Server.notifyDeviceSubscription(devices[i], options, "WL.Server.notifyAllDevices", wasDeprecationWarningMessagePrinted);
        wasDeprecationWarningMessagePrinted = true;
    }
}

/*******************************************************************************/
WL.Server.notifyDevice = function(userSubscription, device, options) {
    var devices = userSubscription.getDeviceSubscriptions();
    for (var i = 0; i  < devices.length; i++) {
        if (devices[i].device == device) {
            WL.Server.notifyDeviceSubscription(devices[i], options, "WL.Server.notifyDevice", false);
            return;
        }
    }
    WL.Logger.error("Device token " + deviceToken + " not registered for user " + userSubscription.userId + ". Notification not sent.");
}

/*******************************************************************************/
WL.Server.notifyDeviceSubscription = function(deviceSubscription, options, deprecationWarningMessageOriginMethod, wasDeprecationWarningMessagePrinted) {
	var notification;

	if (options.badge) {
		// the old notification structure
		if (!wasDeprecationWarningMessagePrinted) {
			// Print the message only for the first time
			var deprecationWarningMessageOriginMethodToPrint = deprecationWarningMessageOriginMethod || "WL.Server.notifyDeviceSubscription";
			__integration.warnGlobal(deprecationWarningMessageOriginMethodToPrint, "logger.pushNotificationUsingDeprecatedJson", deprecationWarningMessageOriginMethodToPrint, deviceSubscription.eventSourceId);
		}
	    notification = __createNotification(options);
	} else {
		// the new notification structure
		notification = options;
	}
	notification.GCM.payload.alias = deviceSubscription.alias;
	notification.APNS.payload.alias = deviceSubscription.alias;
	if (notification.MPNS.raw && notification.MPNS.raw.payload) {
		notification.MPNS.raw.payload.alias = deviceSubscription.alias;
	}
	if (notification.MPNS.toast) {
		// alias is not a microsoft property for toast messages. If a toast message arrives when the app is 
		// running then the message's properties are passed to the app's code, 
		// in which case the alias is needed so worklight will know which callback to trigger.
		notification.MPNS.toast.alias = deviceSubscription.alias;
	}

	notification.applicationId = deviceSubscription.applicationId;
	notification.platform = deviceSubscription.platform;
	notification.token = deviceSubscription.token;
	notification.eventSourceId = deviceSubscription.eventSourceId;

    return __integration.submitNotification(notification);
}

/*******************************************************************************/
WL.Server.notifyAll = function(eventSource, options) {
	var notification;
	var deprecationWarningMessageOriginMethodToPrint = "WL.Server.notifyAll";

	if (options.badge) {
		// the old notification structure
		__integration.warnGlobal(deprecationWarningMessageOriginMethodToPrint, "logger.pushNotificationUsingDeprecatedJson", deprecationWarningMessageOriginMethodToPrint, eventSource);
	    notification = __createNotification(options);
	} else {
		// the new notification structure
		notification = options;
	}

	return __integration.broadcastNotification(eventSource, notification);
}

/*******************************************************************************/
WL.Server.createDefaultNotification = function(notificationText, badge, payload) {
    var notificationPayload = payload || {};
	return {
		APNS: {
			badge: badge,
			alert: notificationText,
			payload: notificationPayload,
			sound: "",
			actionKey: null
			},
		GCM: {
			alert: notificationText,
			badge: badge,
			payload: notificationPayload
			},
		SMS: {
			text: notificationText
			},
		MPNS: {
			raw: {
				payload: notificationPayload
				},
			toast: null,
			tile: {
				title: notificationText,
				count: badge 
				}
			}
		};
}

// Build the new notification structure from the old notification structure.
// The old notification structure is deprecated, so issue a message to the log.
function __createNotification(options) {
    var notification = WL.Server.createDefaultNotification(options.alert, options.badge, options.payload);
    notification.APNS.sound = options.sound || "";
    notification.GCM.sound = options.sound || "";
    notification.APNS.actionKey = options.activateButtonLabel || null;
    return notification;
}

/*******************************************************************************/
/**
 * Deprecated, use WL.Server.notifyDeviceSubscription() instead
 **/
WL.Server.submitNotification = function(subscription, options) {
    return WL.Server.notifyDeviceSubscription(subscription, options, "WL.Server.submitNotification");
}

/*******************************************************************************/
WL.Server.signSoapMessage = function (envelope, wsId, keystoreAlias) {
    var signedEnvelope = new XML(__integration.signSoapMessage(envelope.toXMLString(), wsId, keystoreAlias));
    delete envelope.*;
    envelope.* = signedEnvelope.*;
}

/*******************************************************************************/
WL.Server.validateSignature = function (envelope) {
    var signedEnvelope = new XML(__integration.validateSignature(envelope.toXMLString()));
}

/*******************************************************************************/
function __getDataAccessService() {
	return __integration.getDataAccessService();
}

/*******************************************************************************/
function __validateInvocationData(invocationData) {
	if (invocationData == null) {
        return __createFailureInvocationResult("InvocationData is null or undefined.");
	}

    var msgPrefix = "Invalid invocation data. ";
	var adapterName = invocationData.adapter;
	if(adapterName == null) {
		return __createFailureInvocationResult(msgPrefix + "Adapter name is null or undefined.\nInvocation data:  " + __toString(invocationData));
	}
	var adapter = __integration.getAdapterManager().getAdapter(adapterName);
	if (adapter == null) {
		error = "Adapter '" + adapterName + "' does not exist.";
		return __createFailureInvocationResult(msgPrefix + "Adapter '" + adapterName + "' does not exist.\nInvocation data:  " + __toString(invocationData));
	}
	var procedureName = invocationData.procedure;
	if(procedureName == null) {
		return __createFailureInvocationResult(msgPrefix + "Procedure name is null or undefined.\nInvocation data:  " + __toString(invocationData));
	}
	var procedure = adapter.getProcedures().get(procedureName);
	if (procedure == null) {
		return __createFailureInvocationResult(msgPrefix + "Procedure '" + procedureName + "' does not exist.\nInvocation data:  " + __toString(invocationData));
	}
}

/*******************************************************************************/
function __toString(obj) {
    return com.worklight.server.integration.api.JSObjectConverter.toFormattedString(obj);
}
/*******************************************************************************/
function __createFailureInvocationResult(message) {
	var res = {};
	res.info = [];
	res.isSuccessful = false;
	res.warnings = [];
	res.errors = [message];
	return res;
}

